#ifndef _VX_CVIRT_DEF_H
#define _VX_CVIRT_DEF_H

#include <linux/jiffies.h>
#include <linux/utsname.h>
#include <linux/spinlock.h>
#include <linux/time.h>
#include <asm/atomic.h>


struct _vx_usage_stat {
	uint64_t user;
	uint64_t nice;
	uint64_t system;
	uint64_t softirq;
	uint64_t irq;
	uint64_t idle;
	uint64_t iowait;
};

/* context sub struct */

struct _vx_cvirt {
	int max_threads;		/* maximum allowed threads */
	atomic_t nr_threads;		/* number of current threads */
	atomic_t nr_running;		/* number of running threads */
	atomic_t nr_uninterruptible;	/* number of uninterruptible threads */

	atomic_t nr_onhold;		/* processes on hold */
	uint32_t onhold_last;		/* jiffies when put on hold */

	struct timespec bias_idle;
	struct timespec bias_uptime;	/* context creation point */

	struct new_utsname utsname;

	spinlock_t load_lock;		/* lock for the load averages */
	atomic_t load_updates;		/* nr of load updates done so far */
	uint32_t load_last;		/* last time load was cacled */
	uint32_t load[3];		/* load averages 1,5,15 */

	struct _vx_usage_stat cpustat[NR_CPUS];
};

struct _vx_sock_acc {
	atomic_t count;
	atomic_t total;
};

/* context sub struct */

struct _vx_cacct {
	unsigned long total_forks;

	struct _vx_sock_acc sock[5][3];
};


#ifdef	_VX_INFO_INIT_

extern uint64_t vx_idle_jiffies(void);

static inline void vx_info_init_cvirt(struct _vx_cvirt *cvirt)
{
	uint64_t idle_jiffies = vx_idle_jiffies();

	do_posix_clock_monotonic_gettime(&cvirt->bias_uptime);
	jiffies_to_timespec(idle_jiffies, &cvirt->bias_idle);
	atomic_set(&cvirt->nr_threads, 0);
	atomic_set(&cvirt->nr_running, 0);
	atomic_set(&cvirt->nr_uninterruptible, 0);
	atomic_set(&cvirt->nr_onhold, 0);

	down_read(&uts_sem);
	cvirt->utsname = system_utsname;
	up_read(&uts_sem);

	spin_lock_init(&cvirt->load_lock);
	cvirt->load_last = jiffies;
	atomic_set(&cvirt->load_updates, 0);
	cvirt->load[0] = 0;
	cvirt->load[1] = 0;
	cvirt->load[2] = 0;
}

static inline void vx_info_exit_cvirt(struct _vx_cvirt *cvirt)
{
#ifdef	CONFIG_VSERVER_DEBUG
	int value;

	if ((value = atomic_read(&cvirt->nr_threads)))
		printk("!!! cvirt: %p[nr_threads] = %d on exit.\n",
			cvirt, value);
	if ((value = atomic_read(&cvirt->nr_running)))
		printk("!!! cvirt: %p[nr_running] = %d on exit.\n",
			cvirt, value);
	if ((value = atomic_read(&cvirt->nr_uninterruptible)))
		printk("!!! cvirt: %p[nr_uninterruptible] = %d on exit.\n",
			cvirt, value);
#endif
	return;
}

static inline void vx_info_init_cacct(struct _vx_cacct *cacct)
{
	int i,j;

	for (i=0; i<5; i++) {
		for (j=0; j<3; j++) {
			atomic_set(&cacct->sock[i][j].count, 0);
			atomic_set(&cacct->sock[i][j].total, 0);
		}
	}
}

static inline void vx_info_exit_cacct(struct _vx_cacct *cacct)
{
	return;
}

#endif	/* _VX_INFO_INIT_ */
#endif	/* _VX_CVIRT_DEF_H */
